import datasets
from datasets import load_dataset, load_from_disk
from PIL import Image, ImageOps, ImageChops
import random
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import math, requests, argparse, os, gc, time, traceback
from diffusers.utils import make_image_grid



def adjust_size(image, target_size=(512,512)):
    mask_image = Image.new("RGB", image.size, (255, 255, 255))
    nmask_image = Image.new("RGB", image.size, (0, 0, 0))
    new_image = ImageOps.pad(image, target_size, Image.LANCZOS, (255, 255, 255), (0.5,0.5))
    mask_image = ImageOps.pad(mask_image, target_size, Image.LANCZOS, (100, 100, 100), (0.5,0.5))
    nmask_image = ImageOps.pad(nmask_image, target_size, Image.LANCZOS, (100, 100, 100), (0.5,0.5))
    return new_image, mask_image, nmask_image


def save_image(path, image):
    directory = os.path.dirname(path)
    if not os.path.exists(directory):
        os.makedirs(directory)
    image.save(path)


def aspect_ratio(size, threshold):
    if size[0] < 512:
        return false
    if size[1] < 512:
        return false
    large_side = size[0] if size[0] >= size[1] else size[1]
    small_side = size[0] if size[0] < size[1] else size[1]
    return (large_side/small_side <= threshold)



if __name__ == '__main__':

    parser = argparse.ArgumentParser()

    parser.add_argument(
        "-i",
		"--input",
		type=str,
        default="wikiart/train",
		required=False,
		help="input images dataset path",
	)

    parser.add_argument(
        "-l",
		"--length",
		type=int,
		required=True,
		help="num images",
	)

    parser.add_argument(
        "-grows",
		"--gridrows",
		type=int,
		required=True,
		help="grid rows",
	)

    parser.add_argument(
        "-gcols",
		"--gridcols",
		type=int,
		required=True,
		help="grid cols",
	)

    parser.add_argument(
        "-s",
		"--seed",
		type=int,
        default=0,
		required=False,
		help="seed",
	)

    parser.add_argument(
        "-oi",
		"--outputimage",
		type=str,
		required=True,
		help="destination path",
	)

    parser.add_argument(
        "-oim",
		"--outputimagesmetadata",
		type=str,
        default="./wikiart",
		required=False,
		help="destination path",
	)

    parser.add_argument(
		"--save-metadata",
		action='store_true',
		required=False,
		help="destination path",
	)
    
    parser.add_argument(
        "-o",
		"--output",
		type=str,
        default="wikiart_test",
		required=False,
		help="destination path",
	)

    args = parser.parse_args()
    
    wikiart_info = {"artist": ["Unknown Artist", "boris-kustodiev", "camille-pissarro", "childe-hassam", "claude-monet", "edgar-degas", "eugene-boudin", "gustave-dore", "ilya-repin", "ivan-aivazovsky", "ivan-shishkin", "john-singer-sargent", "marc-chagall", "martiros-saryan", "nicholas-roerich", "pablo-picasso", "paul-cezanne", "pierre-auguste-renoir", "pyotr-konchalovsky", "raphael-kirchner", "rembrandt", "salvador-dali", "vincent-van-gogh", "hieronymus-bosch", "leonardo-da-vinci", "albrecht-durer", "edouard-cortes", "sam-francis", "juan-gris", "lucas-cranach-the-elder", "paul-gauguin", "konstantin-makovsky", "egon-schiele", "thomas-eakins", "gustave-moreau", "francisco-goya", "edvard-munch", "henri-matisse", "fra-angelico", "maxime-maufra", "jan-matejko", "mstislav-dobuzhinsky", "alfred-sisley", "mary-cassatt", "gustave-loiseau", "fernando-botero", "zinaida-serebriakova", "georges-seurat", "isaac-levitan", "joaqu\u00e3\u00adn-sorolla", "jacek-malczewski", "berthe-morisot", "andy-warhol", "arkhip-kuindzhi", "niko-pirosmani", "james-tissot", "vasily-polenov", "valentin-serov", "pietro-perugino", "pierre-bonnard", "ferdinand-hodler", "bartolome-esteban-murillo", "giovanni-boldini", "henri-martin", "gustav-klimt", "vasily-perov", "odilon-redon", "tintoretto", "gene-davis", "raphael", "john-henry-twachtman", "henri-de-toulouse-lautrec", "antoine-blanchard", "david-burliuk", "camille-corot", "konstantin-korovin", "ivan-bilibin", "titian", "maurice-prendergast", "edouard-manet", "peter-paul-rubens", "aubrey-beardsley", "paolo-veronese", "joshua-reynolds", "kuzma-petrov-vodkin", "gustave-caillebotte", "lucian-freud", "michelangelo", "dante-gabriel-rossetti", "felix-vallotton", "nikolay-bogdanov-belsky", "georges-braque", "vasily-surikov", "fernand-leger", "konstantin-somov", "katsushika-hokusai", "sir-lawrence-alma-tadema", "vasily-vereshchagin", "ernst-ludwig-kirchner", "mikhail-vrubel", "orest-kiprensky", "william-merritt-chase", "aleksey-savrasov", "hans-memling", "amedeo-modigliani", "ivan-kramskoy", "utagawa-kuniyoshi", "gustave-courbet", "william-turner", "theo-van-rysselberghe", "joseph-wright", "edward-burne-jones", "koloman-moser", "viktor-vasnetsov", "anthony-van-dyck", "raoul-dufy", "frans-hals", "hans-holbein-the-younger", "ilya-mashkov", "henri-fantin-latour", "m.c.-escher", "el-greco", "mikalojus-ciurlionis", "james-mcneill-whistler", "karl-bryullov", "jacob-jordaens", "thomas-gainsborough", "eugene-delacroix", "canaletto"], "genre": ["abstract_painting", "cityscape", "genre_painting", "illustration", "landscape", "nude_painting", "portrait", "religious_painting", "sketch_and_study", "still_life", "Unknown Genre"], "style": ["Abstract_Expressionism", "Action_painting", "Analytical_Cubism", "Art_Nouveau", "Baroque", "Color_Field_Painting", "Contemporary_Realism", "Cubism", "Early_Renaissance", "Expressionism", "Fauvism", "High_Renaissance", "Impressionism", "Mannerism_Late_Renaissance", "Minimalism", "Naive_Art_Primitivism", "New_Realism", "Northern_Renaissance", "Pointillism", "Pop_Art", "Post_Impressionism", "Realism", "Rococo", "Romanticism", "Symbolism", "Synthetic_Cubism", "Ukiyo_e"]}

    if not os.path.exists(args.input):
        dataset = load_dataset("huggan/wikiart")
        dataset.save_to_disk('wikiart')
    else:
        dataset = load_from_disk(args.input)
    dataset = dataset.shuffle(seed=args.seed)
    small_dataset = dataset.select(range(10000))
    grid = []
    indices = []
    cnt = 1
    for i,img in enumerate(small_dataset['image']):
        if aspect_ratio(img.size, 1.2):
            image, mask, nmask = adjust_size(img)
            grid.append(image)
            cnt += 1
            indices.append(i)
        if cnt > args.length:
            break
    
    save_image(args.outputimage, make_image_grid(grid, args.gridrows, args.gridcols))
    small_dataset = dataset.select(indices)
    small_dataset.save_to_disk(args.output)
    if args.save_metadata:
        df = pd.DataFrame(small_dataset)
        df = df.drop(['image'], axis=1)
        df.insert(loc=0, column='image_id', value=df.index)
        df['artist'] = df['artist'].map(lambda x: wikiart_info['artist'][x])
        df['genre'] = df['genre'].map(lambda x: wikiart_info['genre'][x])
        df['style'] = df['style'].map(lambda x: wikiart_info['style'][x])
        #df.to_csv(f"results/imgs_metadata.csv", index=False)
        df.to_excel(f"{args.outputimagesmetadata}/imgs_metadata.xlsx", index=False, engine="openpyxl")
